/* ==================================== JUCER_BINARY_RESOURCE ====================================

   This is an auto-generated file: Any edits you make may be overwritten!

*/

#include <cstring>

namespace BinaryData
{

//================== transpose_chevron@2x.png ==================
static const unsigned char temp_binary_data_7[] =
"\x89PNG\r\n"
"\x1a\n"
"\0\0\0\rIHDR\0\0\0\x0e\0\0\0\x06\x08\x06\0\0\0\xf3\x1b\xaf\xbc\0\0\0\tpHYs\0\0\x16%\0\0\x16%\x01IR$\xf0\0\0\x06,iTXtXML:com.adobe.xmp\0\0\0\0\0<?xpacket begin=\"\xef\xbb\xbf\" id=\"W5M0MpCehiHzreSzNTczkc9d\"?> <x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:"
"xmptk=\"Adobe XMP Core 10.0-c000 79.d04cc1698, 2025/07/02-12:18:13        \"> <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"> <rdf:Description rdf:about=\"\" xmlns:exif=\"http://ns.adobe.com/exif/1.0/\" xmlns:xmp=\"http://ns.adobe"
".com/xap/1.0/\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:photoshop=\"http://ns.adobe.com/photoshop/1.0/\" xmlns:xmpMM=\"http://ns.adobe.com/xap/1.0/mm/\" xmlns:stEvt=\"http://ns.adobe.com/xap/1.0/sType/ResourceEvent#\" exif:PixelXDimension"
"=\"64\" exif:PixelYDimension=\"32\" exif:UserComment=\"Screenshot\" xmp:CreateDate=\"2026-03-03T07:04:56-05:00\" xmp:ModifyDate=\"2026-03-03T07:06:52-05:00\" xmp:MetadataDate=\"2026-03-03T07:06:52-05:00\" dc:format=\"image/png\" photoshop:ColorMode=\""
"3\" xmpMM:InstanceID=\"xmp.iid:e38fd1c7-c4fb-4be3-b24e-531d5c43f379\" xmpMM:DocumentID=\"adobe:docid:photoshop:3af437ed-46eb-4948-adf1-a66b1c5cce83\" xmpMM:OriginalDocumentID=\"xmp.did:7aa9bd4b-eec6-4337-b239-b7f6320378a5\"> <xmpMM:History> <rdf:Seq>"
" <rdf:li stEvt:action=\"saved\" stEvt:instanceID=\"xmp.iid:7aa9bd4b-eec6-4337-b239-b7f6320378a5\" stEvt:when=\"2026-03-03T07:06:52-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.3 (Macintosh)\" stEvt:changed=\"/\"/> <rdf:li stEvt:action=\"saved\" s"
"tEvt:instanceID=\"xmp.iid:e38fd1c7-c4fb-4be3-b24e-531d5c43f379\" stEvt:when=\"2026-03-03T07:06:52-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.3 (Macintosh)\" stEvt:changed=\"/\"/> </rdf:Seq> </xmpMM:History> </rdf:Description> </rdf:RDF> </x:xmp"
"meta> <?xpacket end=\"r\"?>W#\xaa\x99\0\0\0<IDAT\x18\x19""cp\xa8\xd2\xfd\x9fr\xd3\x81$\x0c\xd2\xc3\xf0\xff\xff\x7f\x06R4\x83\xd4\x82\xf4\x80""5\x12\xab\x19\xa6\tE#!\xcd\xc8\x9a""04\xe2\xd2\x8c\xae\t\xab""Ft\xcd\xd8""4\x81""4\x02\0\xf0\x1c\xd8\xc7_\xcf"
"W\xdb\0\0\0\0IEND\xae""B`\x82";

const char* transpose_chevron2x_png = (const char*) temp_binary_data_7;
}
