/* ==================================== JUCER_BINARY_RESOURCE ====================================

   This is an auto-generated file: Any edits you make may be overwritten!

*/

#include <cstring>

namespace BinaryData
{

//================== slider_handle@2x.png ==================
static const unsigned char temp_binary_data_9[] =
"\x89PNG\r\n"
"\x1a\n"
"\0\0\0\rIHDR\0\0\0\x1a\0\0\0\x10\x08\x06\0\0\0\x08\xd1o\xa8\0\0\0\tpHYs\0\0\x16%\0\0\x16%\x01IR$\xf0\0\0\x06.iTXtXML:com.adobe.xmp\0\0\0\0\0<?xpacket begin=\"\xef\xbb\xbf\" id=\"W5M0MpCehiHzreSzNTczkc9d\"?> <x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmp"
"tk=\"Adobe XMP Core 10.0-c000 79.d20e46630, 2025/12/09-02:11:23        \"> <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"> <rdf:Description rdf:about=\"\" xmlns:xmp=\"http://ns.adobe.com/xap/1.0/\" xmlns:dc=\"http://purl.org/dc/el"
"ements/1.1/\" xmlns:photoshop=\"http://ns.adobe.com/photoshop/1.0/\" xmlns:xmpMM=\"http://ns.adobe.com/xap/1.0/mm/\" xmlns:stEvt=\"http://ns.adobe.com/xap/1.0/sType/ResourceEvent#\" xmlns:exif=\"http://ns.adobe.com/exif/1.0/\" xmp:CreateDate=\"2026-0"
"3-07T11:47:10-05:00\" xmp:ModifyDate=\"2026-03-26T19:56:02-04:00\" xmp:MetadataDate=\"2026-03-26T19:56:02-04:00\" dc:format=\"image/png\" photoshop:ColorMode=\"3\" xmpMM:InstanceID=\"xmp.iid:47fd88be-7321-4c14-ac3d-6b36b82bc76b\" xmpMM:DocumentID=\"a"
"dobe:docid:photoshop:0a135e9f-10f7-6e4f-842b-7c83d21dc390\" xmpMM:OriginalDocumentID=\"xmp.did:dd403cd3-8ce7-4e77-82a0-3b0301383444\" exif:PixelXDimension=\"220\" exif:PixelYDimension=\"224\" exif:UserComment=\"Screenshot\"> <xmpMM:History> <rdf:Seq>"
" <rdf:li stEvt:action=\"saved\" stEvt:instanceID=\"xmp.iid:dd403cd3-8ce7-4e77-82a0-3b0301383444\" stEvt:when=\"2026-03-07T15:42:49-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.4 (Macintosh)\" stEvt:changed=\"/\"/> <rdf:li stEvt:action=\"saved\" s"
"tEvt:instanceID=\"xmp.iid:47fd88be-7321-4c14-ac3d-6b36b82bc76b\" stEvt:when=\"2026-03-26T19:56:02-04:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.4 (Macintosh)\" stEvt:changed=\"/\"/> </rdf:Seq> </xmpMM:History> </rdf:Description> </rdf:RDF> </x:xmp"
"meta> <?xpacket end=\"r\"?>\xf6\xed\xa0\xc0\0\0\0""AIDAT8\x11""c\xf8\xff\xff?\x03\x10\xfc\xa7%\x86\xda""A[K\x90""0\xc3\xff\xec\xdc,8vpr\xa0\n"
"\x16\x15\x17\x83\xe3Q\x8b\x86\x90""E\xff\xfe\xfd\xa3)\xa6\xbf""E\xa3\x89""a\xd4\"\xb8""Et\xc1\xf4\xaa&\0)\xed\xeb""7\xd1\xe6""9-\0\0\0\0IEND\xae""B`\x82";

const char* slider_handle2x_png = (const char*) temp_binary_data_9;
}
