/* ==================================== JUCER_BINARY_RESOURCE ====================================

   This is an auto-generated file: Any edits you make may be overwritten!

*/

#include <cstring>

namespace BinaryData
{

//================== led_red@2x.png ==================
static const unsigned char temp_binary_data_5[] =
"\x89PNG\r\n"
"\x1a\n"
"\0\0\0\rIHDR\0\0\0\x12\0\0\0$\x08\x06\0\0\0\x84\xedj\xe7\0\0\0\tpHYs\0\0\n"
"\xeb\0\0\n"
"\xeb\x01\x82\x8b\rZ\0\0\x06\xa1iTXtXML:com.adobe.xmp\0\0\0\0\0<?xpacket begin=\"\xef\xbb\xbf\" id=\"W5M0MpCehiHzreSzNTczkc9d\"?> <x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 10.0-c000 79.d04cc1698, 2025/07/02-12:18:13        \"> <rd"
"f:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"> <rdf:Description rdf:about=\"\" xmlns:xmp=\"http://ns.adobe.com/xap/1.0/\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:photoshop=\"http://ns.adobe.com/photoshop/1.0/\" xmlns:xmp"
"MM=\"http://ns.adobe.com/xap/1.0/mm/\" xmlns:stEvt=\"http://ns.adobe.com/xap/1.0/sType/ResourceEvent#\" xmp:CreatorTool=\"Adobe Photoshop 27.3 (Macintosh)\" xmp:CreateDate=\"2026-03-01T16:47:47-05:00\" xmp:ModifyDate=\"2026-03-02T13:05:30-05:00\" xmp"
":MetadataDate=\"2026-03-02T13:05:30-05:00\" dc:format=\"image/png\" photoshop:ColorMode=\"3\" xmpMM:InstanceID=\"xmp.iid:395e74ab-f539-45a5-855b-ad501fdd67c1\" xmpMM:DocumentID=\"adobe:docid:photoshop:d0949b65-fa1e-4843-865b-dfcb684a8419\" xmpMM:Orig"
"inalDocumentID=\"xmp.did:f04fef00-21e7-40d5-8d90-830dc10e252f\"> <xmpMM:History> <rdf:Seq> <rdf:li stEvt:action=\"created\" stEvt:instanceID=\"xmp.iid:f04fef00-21e7-40d5-8d90-830dc10e252f\" stEvt:when=\"2026-03-01T16:47:47-05:00\" stEvt:softwareAgent"
"=\"Adobe Photoshop 27.3 (Macintosh)\"/> <rdf:li stEvt:action=\"saved\" stEvt:instanceID=\"xmp.iid:859237ce-bd5b-4a2c-bcae-51e9752fd531\" stEvt:when=\"2026-03-02T12:55:44-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.3 (Macintosh)\" stEvt:changed=\""
"/\"/> <rdf:li stEvt:action=\"saved\" stEvt:instanceID=\"xmp.iid:395e74ab-f539-45a5-855b-ad501fdd67c1\" stEvt:when=\"2026-03-02T13:05:30-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.3 (Macintosh)\" stEvt:changed=\"/\"/> </rdf:Seq> </xmpMM:History>"
" </rdf:Description> </rdf:RDF> </x:xmpmeta> <?xpacket end=\"r\"?>Bl\x8b\xe7\0\0\0\xc8IDATH\xc7\xe5\x94=\x0e\x83""0\x0c""Fs\x02$$f\xeeP1\x01\xed\xc4P\x18\xd8\xaa\x8a\x85\xa1\xd7\xab\xc4\x91z\x14\xea&\x0e_\x87(\x81%\xa9,ux\x83\xf5YO\xce\xaf\"\"\x15\x03"
"\x95LTU\x15\x19\x94\x8e| O/r\x05\x8f\xb6\xf5\x12\x12zE\xa7\xb2\x0c\x8aL\xb6+r'y-\x0bs4\xd9\xef""DG\xfc\xa3(\xda\xf1G\xbb\x90\xf2\xde\x9a\xdc\xffH\x9e\xe8\xde""4d8g\x19s\xcds\x06""5\xf2\xf4\"4\xdc\x8a\x82\xa1y\xb6L\x93""e\xab\x91\xbb""B\xaf\xe8Y\xd7""A"
"\x91\xc9vE\x17=\xb6\xe1\xad\x9b\x99qd\xd6\xbegP#G\x7f:\x11""6\x15KY\x87\xc1\xd2u\x96\xad""F\xfe\xedO&\x92\xb7G\xd1\x8e?\xda\x85\x94\xf7\xd6\xc4\xfcG\x1f\x04\x05\xe3\xf0\xb7t\xde\xaf\0\0\0\0IEND\xae""B`\x82";

const char* led_red2x_png = (const char*) temp_binary_data_5;
}
