/* ==================================== JUCER_BINARY_RESOURCE ====================================

   This is an auto-generated file: Any edits you make may be overwritten!

*/

#include <cstring>

namespace BinaryData
{

//================== switch_3way@2x.png ==================
static const unsigned char temp_binary_data_3[] =
"\x89PNG\r\n"
"\x1a\n"
"\0\0\0\rIHDR\0\0\0\x12\0\0\0\x90\x08\x06\0\0\0\x05\xb6;\x84\0\0\0\tpHYs\0\0\x0b\x13\0\0\x0b\x13\x01\0\x9a\x9c\x18\0\0\x06\xa1iTXtXML:com.adobe.xmp\0\0\0\0\0<?xpacket begin=\"\xef\xbb\xbf\" id=\"W5M0MpCehiHzreSzNTczkc9d\"?> <x:xmpmeta xmlns:x=\"adobe:"
"ns:meta/\" x:xmptk=\"Adobe XMP Core 10.0-c000 79.d20e46630, 2025/12/09-02:11:23        \"> <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"> <rdf:Description rdf:about=\"\" xmlns:xmp=\"http://ns.adobe.com/xap/1.0/\" xmlns:dc=\"http:"
"//purl.org/dc/elements/1.1/\" xmlns:photoshop=\"http://ns.adobe.com/photoshop/1.0/\" xmlns:xmpMM=\"http://ns.adobe.com/xap/1.0/mm/\" xmlns:stEvt=\"http://ns.adobe.com/xap/1.0/sType/ResourceEvent#\" xmp:CreatorTool=\"Adobe Photoshop 27.3 (Macintosh)\""
" xmp:CreateDate=\"2026-03-01T16:47:47-05:00\" xmp:ModifyDate=\"2026-03-06T23:05:20-05:00\" xmp:MetadataDate=\"2026-03-06T23:05:20-05:00\" dc:format=\"image/png\" photoshop:ColorMode=\"3\" xmpMM:InstanceID=\"xmp.iid:8582b184-d45b-41b2-981e-5b292094137"
"4\" xmpMM:DocumentID=\"adobe:docid:photoshop:f9b13558-787a-c549-a06e-a6d43716950e\" xmpMM:OriginalDocumentID=\"xmp.did:6d00d7c8-ab81-4b49-8884-cc7b7b90b91e\"> <xmpMM:History> <rdf:Seq> <rdf:li stEvt:action=\"created\" stEvt:instanceID=\"xmp.iid:6d00d"
"7c8-ab81-4b49-8884-cc7b7b90b91e\" stEvt:when=\"2026-03-01T16:47:47-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.3 (Macintosh)\"/> <rdf:li stEvt:action=\"saved\" stEvt:instanceID=\"xmp.iid:67c41176-1c83-4415-8b21-8d1df79463c9\" stEvt:when=\"2026-0"
"3-02T12:54:52-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.3 (Macintosh)\" stEvt:changed=\"/\"/> <rdf:li stEvt:action=\"saved\" stEvt:instanceID=\"xmp.iid:8582b184-d45b-41b2-981e-5b2920941374\" stEvt:when=\"2026-03-06T23:05:20-05:00\" stEvt:softw"
"areAgent=\"Adobe Photoshop 27.4 (Macintosh)\" stEvt:changed=\"/\"/> </rdf:Seq> </xmpMM:History> </rdf:Description> </rdf:RDF> </x:xmpmeta> <?xpacket end=\"r\"?>!m\xa9\xc5\0\0\x01ZIDATh\xde\xed\x9a\x8b\r\x83""0\x0c""DY\x80\x01\x98\x83""5\x98\x83""9X\x83"
"9X\x83""9X\x83""6\x96\xae\x02\x8b\x14\xd3\xba\x91\x8b.\xd2IE\x17?E|\xaa""8vU\xadk\xb5U\xd3""4kRn\xc0\xd7q\xd5\x11h\x18\x86,(yoAg+9]\xd9\x16T\xd7\xb5\x19\x94\xe6\xee@ 'C\xb0\xc6\x91\xe6\x02&\xc0\xd7\x8f\xa7q\x15\x94\xf4G\xa0""eYD\xb9\x01\xbf\x1ch\x9eg"
"\x11\x02\xb4\xe0\x97\x03M\xd3$B\x80\x16\xfcr\xa0q\x1c""E\x08\xd0\x82_\x0e\x94\xfe&\x92\x10\xa0\x05\xbf\x1c\xa8\xef{\x11\x02\xb4\xe0\x97\x03u]'B\x80\x16\xfcr\xa0\xb6mE\x08\xd0\x82_\x0e\x84k\x04h\xe5\xe6\xff\x0e\xa4'h\xe5\xe6\xfd\x1e""d\x15""A\x04\x11""D"
"\x90\xda\x1e_\xd1n{\xac""7\xecW!\xd9\x14\xc2\x02\xdb\xad\xe4]Rc\x91);\xba\x19\xc8\xed""f\x7f\xfd\xf8\xdd^H~\xfd\x04\x11t\x13\xd0\xd1""a\xd4""6\xe1""3\x81\xac\xe7\"\xa7\xa9\xe8\xd9""0\xa5\xa2""7>?r\x03\xb9\xddl\xb7\xc7\xef\xfa""B\xf2\xeb'\x88 \x82\x98"
"\xaf""1\xcd""b\x9a""E\x10""A\x04\x11\x14\xb4\xc0\x12\xaf\x08\x15\xafP\x17\xaf\x98\x19\xaf\xe0\x1b\xaf(\x1e\xafq ^sE\xbc\x06\x94xM:7<\x88r\xeb\x1a\xfb\xb4\x8f\xed""0_s\xeb\xac\x83,\xbd~\xe6|\xcd\xa5\xfb\xf0\x13\xd0\x03;\xf0y\x05\x81""E\x06\xa1\0\0\0\0"
"IEND\xae""B`\x82";

const char* switch_3way2x_png = (const char*) temp_binary_data_3;
}
