/* ==================================== JUCER_BINARY_RESOURCE ====================================

   This is an auto-generated file: Any edits you make may be overwritten!

*/

#include <cstring>

namespace BinaryData
{

//================== switch_3way_horizontal@2x.png ==================
static const unsigned char temp_binary_data_4[] =
"\x89PNG\r\n"
"\x1a\n"
"\0\0\0\rIHDR\0\0\0""0\0\0\0""6\x08\x06\0\0\0\x81[\x1a\x9a\0\0\0\tpHYs\0\0\x0b\x13\0\0\x0b\x13\x01\0\x9a\x9c\x18\0\0\x06\xa1iTXtXML:com.adobe.xmp\0\0\0\0\0<?xpacket begin=\"\xef\xbb\xbf\" id=\"W5M0MpCehiHzreSzNTczkc9d\"?> <x:xmpmeta xmlns:x=\"adobe:ns"
":meta/\" x:xmptk=\"Adobe XMP Core 10.0-c000 79.d20e46630, 2025/12/09-02:11:23        \"> <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"> <rdf:Description rdf:about=\"\" xmlns:xmp=\"http://ns.adobe.com/xap/1.0/\" xmlns:dc=\"http://"
"purl.org/dc/elements/1.1/\" xmlns:photoshop=\"http://ns.adobe.com/photoshop/1.0/\" xmlns:xmpMM=\"http://ns.adobe.com/xap/1.0/mm/\" xmlns:stEvt=\"http://ns.adobe.com/xap/1.0/sType/ResourceEvent#\" xmp:CreatorTool=\"Adobe Photoshop 27.4 (Macintosh)\" x"
"mp:CreateDate=\"2026-03-01T16:47:47-05:00\" xmp:ModifyDate=\"2026-03-06T23:03:53-05:00\" xmp:MetadataDate=\"2026-03-06T23:03:53-05:00\" dc:format=\"image/png\" photoshop:ColorMode=\"3\" xmpMM:InstanceID=\"xmp.iid:99a30359-4f83-4c03-971d-27cc11fcafaa\""
" xmpMM:DocumentID=\"adobe:docid:photoshop:8d6afce7-759d-b449-bb99-41cda0d59efa\" xmpMM:OriginalDocumentID=\"xmp.did:21de2b4b-687b-4393-b809-40970c9c9dd6\"> <xmpMM:History> <rdf:Seq> <rdf:li stEvt:action=\"created\" stEvt:instanceID=\"xmp.iid:21de2b4b"
"-687b-4393-b809-40970c9c9dd6\" stEvt:when=\"2026-03-01T16:47:47-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.4 (Macintosh)\"/> <rdf:li stEvt:action=\"saved\" stEvt:instanceID=\"xmp.iid:3dca9ac1-bd69-421a-96f5-08aaff584161\" stEvt:when=\"2026-03-0"
"6T17:59:54-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.4 (Macintosh)\" stEvt:changed=\"/\"/> <rdf:li stEvt:action=\"saved\" stEvt:instanceID=\"xmp.iid:99a30359-4f83-4c03-971d-27cc11fcafaa\" stEvt:when=\"2026-03-06T23:03:53-05:00\" stEvt:software"
"Agent=\"Adobe Photoshop 27.4 (Macintosh)\" stEvt:changed=\"/\"/> </rdf:Seq> </xmpMM:History> </rdf:Description> </rdf:RDF> </x:xmpmeta> <?xpacket end=\"r\"?>\x1bVS\x98\0\0\x01 IDATh\xde\xed\x99\xeb\r\x83""0\x0c\x84\xb3\0\x03""0\x07k0G\xe6`\r\xe6`\r\xe6"
"`\rW\xb6p\x14\xf1P\x91\x88\xdb""8:K\xd7\x1f\xa8\x95\xfc""A\x1b\xdf\xb9!\x10\x85\\}\xdfS\xcd:\xf6\x1b\x9a\x01\xd0""7t]'Wk\x14\xf7v\x02\xc9\x01jn\xfe\x08\x91\0\x94\xe8\xae\xb6m\x13\xad\xeb*Z\x96""E4\xcf\xb3h\x9a&Q\x8cQ4\x8e\xa3h\x18\x06Q\xba""cVO\x82_\xb8"
"\x01O\0\xac\xec\xab\xf4\xacj\x03\x01\0\0\0\xd0\n"
"\x80\xfb""c\xd4\xfd \xe3qL\x0e\xad\x04y7ste\xa7\xc9\x81\x9d\xa6\xbdO\xb7y\x80\x9a\r4HdHd/\x13Y\xa9\xe6\xaf\xa6:_3Md\xa5&\xe5\xb7""2\xb5\x12\x16w\xde\xf2I\x14\x07xZ\0\0@\xab\0\xee\x7f\xc4\x7f?F\xdd\x0e""2\xf7V\xa2\x19""3\x87<\0\0$2$\xb2""6\x12\xd9qR\xea"
"\x8aQW\x8e\xba\x82\xd4\x95\xa4\xae(ue\xa9+L]i\xde\x95\x89\x95\xf8%\0\x7f\xd6\x04\xc0\xba\xf1\x93\xbf\x02\0\0\0\0\x80:\x8eQ\xb7\x83\xcc\xab\x95H\xff\xd4 \x91!\x0f\xbc\x03\xf8\0\x9b\xd0wp\x0b\xfe""7\xc1\0\0\0\0IEND\xae""B`\x82";

const char* switch_3way_horizontal2x_png = (const char*) temp_binary_data_4;
}
