/* ==================================== JUCER_BINARY_RESOURCE ====================================

   This is an auto-generated file: Any edits you make may be overwritten!

*/

#include <cstring>

namespace BinaryData
{

//================== kr106_bender_gradient@2x.png ==================
static const unsigned char temp_binary_data_6[] =
"\x89PNG\r\n"
"\x1a\n"
"\0\0\0\rIHDR\0\0\0""d\0\0\0\x0c\x08\x03\0\0\0\xa3\x9b\xe8Q\0\0\0rPLTE\0\0\0\x06\x07\x06\x0f\x0e\x0f\x18\x18\x17##\"...:::GGGSTS```mmmxxy\x84\x84\x84\x90\x8f\x8f\x9c\x9c\x9c\xa9\xa9\xa9\xb5\xb5\xb5\xc2\xc1\xc1\xce\xce\xce\xd9\xda\xda\xe4\xe4\xe4\xee\xed"
"\xee\xed\xed\xed\xe4\xe5\xe4\xda\xd9\xd9\xc1\xc2\xc2\xa8\xa8\xa8\x8f\x90\x8f\x83\x83\x83mlmTSSGGF;:;\"##\x18\x17\x18\x0e\x0f\x0f\x06\x06\x06\xff\xff\xff\xec\xdc\x13\x92\0\0\0\tpHYs\0\0\x0b\x13\0\0\x0b\x13\x01\0\x9a\x9c\x18\0\0\x05\xd2iTXtXML:com.adob"
"e.xmp\0\0\0\0\0<?xpacket begin=\"\xef\xbb\xbf\" id=\"W5M0MpCehiHzreSzNTczkc9d\"?> <x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 10.0-c000 79.d04cc1698, 2025/07/02-12:18:13        \"> <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-"
"rdf-syntax-ns#\"> <rdf:Description rdf:about=\"\" xmlns:xmp=\"http://ns.adobe.com/xap/1.0/\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:photoshop=\"http://ns.adobe.com/photoshop/1.0/\" xmlns:xmpMM=\"http://ns.adobe.com/xap/1.0/mm/\" xmlns:st"
"Evt=\"http://ns.adobe.com/xap/1.0/sType/ResourceEvent#\" xmp:CreatorTool=\"Adobe Photoshop 27.3 (Macintosh)\" xmp:CreateDate=\"2026-03-01T16:47:47-05:00\" xmp:ModifyDate=\"2026-03-02T12:55:59-05:00\" xmp:MetadataDate=\"2026-03-02T12:55:59-05:00\" dc:"
"format=\"image/png\" photoshop:ColorMode=\"2\" xmpMM:InstanceID=\"xmp.iid:aaa635a0-fc68-4412-92fe-a156617e215b\" xmpMM:DocumentID=\"adobe:docid:photoshop:46303651-16af-5242-9b6b-3b7989f8be7d\" xmpMM:OriginalDocumentID=\"xmp.did:b9c2b5ef-41f7-4fd5-8b8"
"f-7749bc325d6a\"> <xmpMM:History> <rdf:Seq> <rdf:li stEvt:action=\"created\" stEvt:instanceID=\"xmp.iid:b9c2b5ef-41f7-4fd5-8b8f-7749bc325d6a\" stEvt:when=\"2026-03-01T16:47:47-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.3 (Macintosh)\"/> <rdf:li"
" stEvt:action=\"saved\" stEvt:instanceID=\"xmp.iid:aaa635a0-fc68-4412-92fe-a156617e215b\" stEvt:when=\"2026-03-02T12:55:59-05:00\" stEvt:softwareAgent=\"Adobe Photoshop 27.3 (Macintosh)\" stEvt:changed=\"/\"/> </rdf:Seq> </xmpMM:History> </rdf:Descri"
"ption> </rdf:RDF> </x:xmpmeta> <?xpacket end=\"r\"?>\xa5\xb6\x89\x81\0\0\0;IDAT8\xcb""cd``$\x01\"\x03\x10\xff\x17\x03;\xc3w\x06.\x86/\x0c\xbc@\xde{\x06!\x86""7\x0c\xa2`\x99\xa7\x0c""2P=w@\xfa""F-\x19\xb5""d\xd4\x92QK\xa8i\t\0""FLT\rX\xf0#y\0\0\0\0IEN"
"D\xae""B`\x82";

const char* kr106_bender_gradient2x_png = (const char*) temp_binary_data_6;
}
