/* =========================================================================================

   This is an auto-generated file: Any edits you make may be overwritten!

*/

#pragma once

namespace BinaryData
{
    extern const char*   kr106_background2x_png;
    const int            kr106_background2x_pngSize = 58552;

    extern const char*   knob2x_png;
    const int            knob2x_pngSize = 4434;

    extern const char*   smallknob2x_png;
    const int            smallknob2x_pngSize = 2882;

    extern const char*   switch_3way2x_png;
    const int            switch_3way2x_pngSize = 2133;

    extern const char*   switch_3way_horizontal2x_png;
    const int            switch_3way_horizontal2x_pngSize = 2075;

    extern const char*   led_red2x_png;
    const int            led_red2x_pngSize = 1987;

    extern const char*   kr106_bender_gradient2x_png;
    const int            kr106_bender_gradient2x_pngSize = 1765;

    extern const char*   transpose_chevron2x_png;
    const int            transpose_chevron2x_pngSize = 1730;

    extern const char*   new_knob2x_png;
    const int            new_knob2x_pngSize = 34984;

    extern const char*   slider_handle2x_png;
    const int            slider_handle2x_pngSize = 1737;

    extern const char*   Segment14_otf;
    const int            Segment14_otfSize = 21872;

    // Number of elements in the namedResourceList and originalFileNames arrays.
    const int namedResourceListSize = 11;

    // Points to the start of a list of resource names.
    extern const char* namedResourceList[];

    // Points to the start of a list of resource filenames.
    extern const char* originalFilenames[];

    // If you provide the name of one of the binary resource variables above, this function will
    // return the corresponding data and its size (or a null pointer if the name isn't found).
    const char* getNamedResource (const char* resourceNameUTF8, int& dataSizeInBytes);

    // If you provide the name of one of the binary resource variables above, this function will
    // return the corresponding original, non-mangled filename (or a null pointer if the name isn't found).
    const char* getNamedResourceOriginalFilename (const char* resourceNameUTF8);
}
